# -*- coding: utf-8 -*-

from rest_framework import serializers
from thx.api.serializers import (
    ThuxCreateSerializerMixin, ThuxDisableSerializerMixin, ThuxEnableSerializerMixin, ThuxListSerializerMixin,
    ThuxPartialUpdateSerializerMixin, ThuxRetrieveSerializerMixin, ThuxUpdateSerializerMixin,
)
from thx.role.core.serializers import ThuxRoleSerializerMixin

from ... import models


# COUNTRY
class CountrySerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_country'
        model = models.Country
        fields = '__all__'


class CountryListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_country_list'
        model = models.Country
        fields = '__all__'


class CountryRetrieveSerializer(ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_country_retrieve'
        model = models.Country
        fields = '__all__'


class CountryCreateSerializer(ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_country_create'
        model = models.Country
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = CountryRetrieveSerializer


class CountryUpdateSerializer(ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_country_update'
        model = models.Country
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = CountryRetrieveSerializer


class CountryPartialUpdateSerializer(ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_country_partial_update'
        model = models.Country
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = CountryRetrieveSerializer


class CountryEnableSerializer(ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_country_enable'
        model = models.Country
        fields = ()
        response_serializer = CountryRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class CountryDisableSerializer(ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_country_disable'
        model = models.Country
        fields = ()
        response_serializer = CountryRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class CountryLightSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_country_light'
        model = models.Country
        fields = ('iso2_code', 'status', 'status_display',)


# ADMINISTRATIVE AREA
class AdministrativeAreaSerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_administrative_area'
        model = models.AdministrativeArea
        fields = '__all__'


class AdministrativeAreaListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    country = CountryRetrieveSerializer()

    class Meta:
        ref_name = 'v1_l10n_administrative_area_list'
        model = models.AdministrativeArea
        fields = '__all__'


class AdministrativeAreaRetrieveSerializer(ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    country = CountryRetrieveSerializer()

    class Meta:
        ref_name = 'v1_l10n_administrative_area_retrieve'
        model = models.AdministrativeArea
        fields = '__all__'


class AdministrativeAreaCreateSerializer(ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_administrative_area_create'
        model = models.AdministrativeArea
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = AdministrativeAreaRetrieveSerializer


class AdministrativeAreaUpdateSerializer(ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_administrative_area_update'
        model = models.AdministrativeArea
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = AdministrativeAreaRetrieveSerializer


class AdministrativeAreaPartialUpdateSerializer(ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_administrative_area_partial_update'
        model = models.AdministrativeArea
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')
        response_serializer = AdministrativeAreaRetrieveSerializer


class AdministrativeAreaEnableSerializer(ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_administrative_area_enable'
        model = models.AdministrativeArea
        fields = ()
        response_serializer = AdministrativeAreaRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance


class AdministrativeAreaDisableSerializer(ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_administrative_area_disable'
        model = models.AdministrativeArea
        fields = ()
        response_serializer = AdministrativeAreaRetrieveSerializer

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance


class AdministrativeAreaLightSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_l10n_administrative_area_light'
        model = models.AdministrativeArea
        fields = ('id', 'status', 'status_display',)
